unit PCF8566;

{$mode objfpc}{$H+}

interface

uses
  Threads,
  Platform,
  RGPIO;



const
  ADR_PCF8566_WR1 : Byte = $7C;




{

GPIO2 -> SDA1 / I2C-Daten
GPIO3 -> SCL1 / I2C-Clock

}



procedure PCF8566_INIT;
procedure PCF8566_Data_Write_Word_Address(b1 : Byte);





implementation






procedure SDA_Set;
begin
  GPIO2_Set;
end;


procedure SCL_Set;
begin
  GPIO3_Set;
end;


procedure SDA_Clear;
begin
  GPIO2_Clear;
end;


procedure SCL_Clear;
begin
  GPIO3_Clear;
end;







procedure PCF8566_INIT;
begin

  SDA_Set;
  SCL_Set;

  GPIO_OUT_Init(GPIO2);
  GPIO_OUT_Init(GPIO3);

  SDA_Set;
  SCL_Set;

end; { procedure PCF8566_INIT }









// Read one Byte from Slave, MSB fist, send NACK
function ds_Shift_In_NACK : Byte;
var
  ds_data, ds_couter : Byte;
begin

  // Data as Input
  GPIO_IN_Init(GPIO2);
  MicrosecondDelay(5);

  ds_data:=0;

  for ds_couter:=0 to 7 do begin

    ds_data:=ds_data shl 1;

    if GPIO2_Read then
      ds_data:=ds_data or %00000001;

    SCL_Set;
    MicrosecondDelay(5);

    SCL_Clear;
    MicrosecondDelay(5);

  end; { for ds_couter:=0 to 7 do }

  // Data as Output
  GPIO_OUT_Init(GPIO2);
  SDA_Set;
  MicrosecondDelay(5);

  SCL_Set;
  MicrosecondDelay(5);

  SCL_Clear;
  MicrosecondDelay(5);

  // Data as Output
  SDA_Clear;

  Result:=ds_data;

end; { function ds_Shift_In_NACK }




// Read one Byte from Slave, MSB fist, send ACK
function ds_Shift_In_ACK : Byte;
var
  ds_data, ds_couter : Byte;
begin

  // Data as Input
  GPIO_IN_Init(GPIO2);
  MicrosecondDelay(5);

  ds_data:=0;

  for ds_couter:=0 to 7 do begin

    ds_data:=ds_data shl 1;

    if GPIO2_Read then
      ds_data:=ds_data or %00000001;

    SCL_Set;
    MicrosecondDelay(5);

    SCL_Clear;
    MicrosecondDelay(5);

  end; { for ds_couter:=0 to 7 do }

  // Data as Output
  SDA_Clear;
  GPIO_OUT_Init(GPIO2);
  SDA_Clear;
  MicrosecondDelay(5);

  SCL_Set;
  MicrosecondDelay(5);

  SCL_Clear;
  MicrosecondDelay(5);

  Result:=ds_data;

end; { function ds_Shift_In_NACK }




function ds_Shift_Out(ds_data : Byte) : Boolean;
var
  ds_couter : Byte;
  ds_result : Boolean;
begin

  // Data as Output
  SDA_Clear;
  GPIO_OUT_Init(GPIO2);
  SDA_Clear;
  MicrosecondDelay(5);

  for ds_couter:=0 to 7 do begin

    if (ds_data and %10000000) = 0 then
      SDA_Clear
    else
      SDA_Set;

    ds_data:=ds_data shl 1;

    MicrosecondDelay(5);

    SCL_Set;
    MicrosecondDelay(5);

    SCL_Clear;
    MicrosecondDelay(5);

  end; { for ds_couter:=0 to 7 do }

  // Data as Input
  GPIO_IN_Init(GPIO2);

  MicrosecondDelay(5);

  SCL_Set;
  MicrosecondDelay(5);

  if GPIO2_Read=false then
    ds_result:=true
  else
    ds_result:=false;

  SCL_Clear;
  MicrosecondDelay(5);

  // Data as Output
  SDA_Clear;
  GPIO_OUT_Init(GPIO2);
  SDA_Clear;
  MicrosecondDelay(5);

  Result:=ds_result;

end; { procedure ds_Shift_Out }




procedure PCF8566_Data_Write_Word_Address(b1 : Byte);
begin

  // Start-Sequenz
  SDA_Clear;
  MicrosecondDelay(5);
  SCL_Clear;
  MicrosecondDelay(5);


  ds_Shift_Out(ADR_PCF8566_WR1);

  // select device at address 000
  ds_Shift_Out(%11100000);

  ds_Shift_Out(%11001001);

  // Address 0
  ds_Shift_Out($00);

  ds_Shift_Out(b1);

  // Stop-Sequenz
  SCL_Set;
  MicrosecondDelay(5);

  SDA_Set;
  MicrosecondDelay(5);

end; { procedure PCF8566_Data_Write_Word_Address }









{==============================================================================}
{==============================================================================}

initialization
 PCF8566_Init;

{==============================================================================}

finalization
 {Nothing}

{==============================================================================}
{==============================================================================}

end.


